/*
 * Beyond Arduino 3 - 2 Guessing_Game.c
 *
 *	Nuts & Volts - Beyond Arduino #3
 *  --------------------------------
 *	A guessing game for 2 players:
 *  Player 1 sets value on potentiometer, presses button PB1
 *  Player 2 guesses value using potentiometer, presses PB2 to guess
 *  Green LED on PB0: Guess too low
 *  Red LED on PD7: Guess too high
 *
 *  Potentiometer connected to PC0
 *
 *  Author: Andrew Retallack
 *			www.crash-bang.com
 */ 

#define F_CPU 16000000UL		//Clock running at 16MHz. Need to define this prior to including "delay.h"

#include <avr/io.h>
#include <util/delay.h>			//Library to handle delays

#define BUTTON_SET PINB1		//Button to set value
#define BUTTON_GUESS PINB2		//Button to guess value
#define LED_LOW PINB0			//LED if guess is low
#define LED_HIGH PIND7			//LED if guess is high

#define SENSITIVITY_RANGE 50	//Set the sensitivity of the game: acceptable +/- range


//Function Prototypes
uint16_t ADC_read(void);
void ADC_init(uint8_t adcChannel);


volatile uint16_t setValue = 0;	//This is the ADC value to be guessed
volatile char valueIsSet = 0;		//Has a value been set?  0=NO

int main(void)
{
	volatile uint16_t currentGuess = 0;	//Current guessed value
	
	
//**CONFIGURE PINS**
	
	//Configure LED pins
	DDRB = (1<<LED_LOW);	//Set LED Low as output; by default all other PORTB pins are an input
	DDRD = (1<<LED_HIGH);	//Set LED High as output
	
	//Set Pull-up resistors on button pins
	PORTB |= (1<<BUTTON_SET) | (1<<BUTTON_GUESS);
	
	//Configure ADC Pin
	DDRC &= ~(1<<0);		//Ensure PC0 is an input
	PORTC &= ~(1<<0);		//Ensure pull-up resistor is not set
	
	
//**Initialise the ADC**
	ADC_init(0);	//Init channel 0
	

	
	
    while(1)
    {
        
		//Has a value been set?
		if (valueIsSet == 0)
		//No value has been set
		{
			//Light both LEDs to show that a value needs to be set
			PORTB |= (1<<LED_LOW);
			PORTD |= (1<<LED_HIGH);
			
			//Is the "Set" button being pressed?
			if ( (PINB & (1<<BUTTON_SET)) == 0)
			//Button is pressed
			{
				
				setValue = ADC_read();	//Store the current value from the ADC
				valueIsSet = 1;	//A value has been set
				
				//Clear the LEDs to indicate the second player needs to guess
				PORTB &= ~(1<<LED_LOW);
				PORTD &= ~(1<<LED_HIGH);
				
			}

		}

		//A Value has been set
		else
		{
			//Is the "Guess" button being pressed?
			if ( (PINB & (1<<BUTTON_GUESS)) == 0)
			//Button is pressed
			{
							
				currentGuess = ADC_read();	//Store the current guess from the ADC
							
				//Clear the LEDs
				PORTB &= ~(1<<LED_LOW);
				PORTD &= ~(1<<LED_HIGH);
				
				
				//Guess is less than the set value
				if (currentGuess < (setValue - SENSITIVITY_RANGE))
				{
					PORTB |= (1<<LED_LOW);	//Set the "too low" LED
				}
				
				//Guess is greater than the value set
				else if (currentGuess > (setValue + SENSITIVITY_RANGE))
				{
					PORTD |= (1<<LED_HIGH);	//Set the "too high" LED
				}
				
				//Guess is within acceptable range, flash the LEDs 6 times
				else
				{
				
					char iCount;
					
					//Start with HIGH LED on, LOW LED off
					PORTD |= (1<<LED_HIGH);	//Set the "too high" LED on
					PORTB &= ~(1<<LED_LOW);	//Set the "too low" LED off
					
					//Loop to toggle both LEDs
					for (iCount = 0; iCount < 12; iCount++)
					{
						_delay_ms(200);
						PORTD ^= (1<<LED_HIGH);	//Toggle LED
						PORTB ^= (1<<LED_LOW);	//Toggle LED
					}
					
					
					valueIsSet = 0;	//Value has been guessed - mark it as unset again
					
				}
					
			
			}

		}		
		
    }
}



/***********************************************************************
*
* This routine initialises the ADC by: 
*   1. setting input channel
*   2. setting prescaler
*   3. enabling ADC
*
* INPUT:	adcChannel The ADC Channel to read from
* RETURNS:	none
************************************************************************/
void ADC_init(uint8_t adcChannel)
{

	ADMUX &= ~( (1<<REFS0) | (1<<REFS1) );	//Reference Voltage = AREF
	ADMUX |= adcChannel;					//The ADC Channel: 0 = ADC0 - 5 = ADC5

	ADCSRA = (1<<ADPS2) | (1<<ADPS1) | (1<<ADPS0);	//Clock Prescaler: 128, allow for accurate (>50kHz <200kHz) sampling with 16MHz system clock

	ADCSRA |= (1<<ADEN);			//Enable the ADC
	
}



/***********************************************************************
*
* This routine takes a reading from the initialised ADC channel
*
* INPUT:	none
* RETURNS:	the value of the reading (0 - 1023)
************************************************************************/
uint16_t ADC_read(void)
{

	uint16_t adcValue;
	
	ADCSRA |= (1<<ADSC);		//Write a "1" to start the conversion.  Bit clears when conversion is complete.
	
	while (ADCSRA & (1<<ADSC));	//Wait for the conversion to finish - ADSC bit goes low on completion
	
	adcValue = ADC;
	
	return adcValue;		//Return the ADC value from 0 to 1023

}

